#include "StdAfx.h"
#include "Entity.h"

CEntity::CEntity(void)
{
	D3DXMatrixIdentity(&m_matrix);
	m_pParent = NULL;
}

CEntity::~CEntity(void)
{
}


D3DXMATRIX CEntity::GetParentMatrix() const
{
	if (!m_pParent)
	{
		D3DXMATRIX m;
		D3DXMatrixIdentity(&m);
		return m;
	}
	return m_pParent->GetMatrix();
}

D3DXMATRIX CEntity::GetMatrix() const
{
	if (m_pParent)
	{
		D3DXMATRIX m;
		D3DXMatrixMultiply(&m, &m_matrix, &GetParentMatrix());
		return m;
	}
	return m_matrix;
}

D3DXVECTOR3 CEntity::GetPos() const
{
	return D3DXVECTOR3(m_matrix._41, m_matrix._42, m_matrix._43);
}

D3DXVECTOR4 CEntity::GetPos4() const
{
	return D3DXVECTOR4(m_matrix._41, m_matrix._42, m_matrix._43, 1);
}

void CEntity::SetMatrix( const D3DXMATRIX& mat )
{
	m_matrix = mat;
}

void CEntity::SetRotation( const D3DXMATRIX& mat )
{
	memcpy(&m_matrix._11, &mat._11, 3*4*sizeof(float));
}

void CEntity::SetPos( const D3DXVECTOR3& pos )
{
	memcpy(&m_matrix.m[3], &pos, 3*sizeof(float));
}

void CEntity::SetParent( CEntity* pParent )
{
	m_pParent = pParent;
}

void CEntity::SetLookAt( D3DXVECTOR3 pos, D3DXVECTOR3 target, D3DXVECTOR3 up )
{
	D3DXMatrixLookAtLH(&m_matrix, &pos, &target, &up);
	D3DXMatrixInverse(&m_matrix, NULL, &m_matrix);
}

D3DXVECTOR3 CEntity::GetUp() const
{
	D3DXVECTOR3 v(m_matrix._21, m_matrix._22, m_matrix._23);
	D3DXVec3TransformCoord(&v, &v, &GetParentMatrix());
	return v;
}

D3DXVECTOR3 CEntity::GetRight() const
{
	D3DXVECTOR3 v(m_matrix._11, m_matrix._12, m_matrix._13);
	D3DXVec3TransformCoord(&v, &v, &GetParentMatrix());
	return v;
}

D3DXVECTOR3 CEntity::GetForward() const
{
	D3DXVECTOR3 v(m_matrix._31, m_matrix._32, m_matrix._33);
	D3DXVec3TransformCoord(&v, &v, &GetParentMatrix());
	return v;
}
