#include "StdAfx.h"
#include "GameLevel.h"
#include "Utils.h"

#include <algorithm>
#include <sstream>
#include <pugixml.hpp>

using namespace std;

CGameLevel::LevelObject::LevelObject( const std::string& sType, D3DXVECTOR2 pos )
: m_strType(sType),
m_pos(pos)
{
}

CGameLevel::CGameLevel(void)
{
}

CGameLevel::~CGameLevel(void)
{
}

bool CGameLevel::Load( const string& strFileName )
{
	//  ,    
	m_vecObjects.clear();

	//   -      
	m_vecObjects.reserve(4096);

	//   
	const string strContents = GetFileAsString(strFileName);
	if (strContents.length()==0)
		return false; // -   

	pugi::xml_document doc;
	pugi::xml_parse_result result = doc.load_buffer_inplace(
		(void*)strContents.c_str(), strContents.length());
	if (!result)
		return false; //    XML

	//  
	for (pugi::xml_node item = doc.first_child(); item; item = item.next_sibling())
	{
		const char* name = item.name();
		if (strcmp(name, "enemy")==0)
		{
			D3DXVECTOR2 pos(0,0);
			
			//  
			stringstream(item.attribute("x").value()) >> pos.x;
			stringstream(item.attribute("y").value()) >> pos.y;
			
			//    
			m_vecObjects.push_back(LevelObject(item.child_value(), pos));
		}
	}
	
	//  
	struct  sort_by_y
	{
		bool operator() (const CGameLevel::LevelObject& a, const CGameLevel::LevelObject& b)
		{ return (a.m_pos.y<b.m_pos.y); }
	} sort_object;
	
	//     Y
	sort(m_vecObjects.begin(), m_vecObjects.end(), sort_object);

	return true;
}

void CGameLevel::GetObjectsInYRange( float fYStart, float fYEnd, LevelObjectsArray& arrOut )
{
	struct filter_by_y
	{
		float m_fYstart, m_fYEnd;
		
		filter_by_y(float fYStart, float fYEnd)
			: m_fYstart(fYStart), m_fYEnd(fYEnd) {}

		bool operator() (const CGameLevel::LevelObject& a)
		{ return (a.m_pos.y>=m_fYstart && a.m_pos.y<m_fYEnd); }
	} filter_object(fYStart, fYEnd);

	copy_if(m_vecObjects.begin(), m_vecObjects.end(), back_inserter(arrOut), filter_object);
}
