#include "StdAfx.h"
#include "Tutorial2D.h"

using namespace std;

CTutorial2D::CTutorial2D(void)
{
	m_pPixelShader = m_pVertexShader = NULL;

	m_pStarField = NULL;
	m_pPlayer = NULL;

	m_keyLeft = 0x25;
	m_keyRight = 0x27;
	m_keyFire = 0x20;
}

CTutorial2D::~CTutorial2D(void)
{
}

bool CTutorial2D::Init()
{
	RECT rc;

	LoadConfig();

	GetClientRect(GetHwnd(), &rc);
	int iWndWidth = rc.right-rc.left;
	int iWndHeight = rc.bottom-rc.top;

	//  
	if (!Graphics::get().Init(GetHwnd(), iWndWidth, iWndHeight))
		return FALSE;

	//   
	m_pVertexShader = Graphics::get().LoadShader("vs.hlsl", "main", "vs_2_0");

	//   
	m_pPixelShader  = Graphics::get().LoadShader("ps.hlsl", "pixel_shader_main", "ps_2_0");

	m_pPlayer = new CPlayerShip();
	m_pStarField = new CStarField();

	Graphics::get().LoadSpriteList("sprites.xml");

	//   
	if (!m_Level.Load("level-01.xml"))
		return FALSE;

	//!  
	if (!InitScripts())
		return FALSE;

	return TRUE;
}

void CTutorial2D::CleanUp()
{
	Graphics::get().Cleanup();
}

void CTutorial2D::Render()
{
	Graphics& gr = Graphics::get();
	if (!FAILED(gr.StartRender(0x00000000)))
	{
		//  -
		gr.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		gr.SetRenderState(D3DRS_LIGHTING, FALSE);
		gr.SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
		gr.SetRenderState(D3DRS_SHADEMODE, D3DSHADE_GOURAUD);
		gr.SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
		gr.SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);
		gr.SetRenderState(D3DRS_ZWRITEENABLE, TRUE);

		D3DXMATRIX mat, matViewProj, matProj, matView;
		D3DXMatrixIdentity(&mat);

		RECT rc;
		GetClientRect(GetHwnd(), &rc);
		int iWndWidth = rc.right-rc.left;
		int iWndHeight = rc.bottom-rc.top;

		//    
		D3DXMatrixTranslation(&matView, 0, 0, 0);
		D3DXMatrixOrthoLH(&matProj, iWndWidth, -iWndHeight, -300, 300);

		//   /
		D3DXMatrixMultiply(&matViewProj, &matView, &matProj);

		//  
		gr.SetPixelShader(m_pPixelShader);
		gr.SetVertexShader(m_pVertexShader);

		//    
		m_pVertexShader->SetMatrix("mWorld", &mat);
		m_pVertexShader->SetMatrix("mViewProjection", &matViewProj);

		//   
		gr.SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
		gr.SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
		gr.SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

		//  -
		gr.SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
		gr.SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
		gr.SetRenderState( D3DRS_DESTBLEND,D3DBLEND_INVSRCALPHA );

		//  
		m_pStarField->Draw();

		//  
		m_pPlayer->Draw();

		//  
		for_each(vecEnemies.begin(), vecEnemies.end(), 
			mem_fun_ref(&CEnemy::Draw));

		//  
		for_each(vecBulletPlayer.begin(), vecBulletPlayer.end(),
			mem_fun_ref(&CBullet::Draw));

		for_each(vecBulletEnemy.begin(), vecBulletEnemy.end(),
			mem_fun_ref(&CBullet::Draw));

		for_each(vecExplosions.begin(), vecExplosions.end(),
			mem_fun(&CExplosion::Draw));

		//  
		gr.EndRender();
	}
}

bool CTutorial2D::LoadConfig()
{
	const std::string config = GetFileAsString("config.xml");
	pugi::xml_document doc;
	pugi::xml_parse_result result = doc.load_buffer_inplace((void*)config.c_str(), config.length());
	if (!result)
		return false; //   

	//    XML-
	pugi::xml_node nodeLeft = doc.child("left");
	pugi::xml_node nodeRight = doc.child("right");
	pugi::xml_node nodeFire = doc.child("fire");

	// load defaults
	m_keyLeft = 0x25;
	m_keyRight = 0x27;
	m_keyFire = 0x20;

	//    XML-
	if (nodeLeft)
		stringstream(nodeLeft.child_value())>>std::hex>>m_keyLeft;
	if (nodeRight)
		stringstream(nodeRight.child_value())>>std::hex>>m_keyRight;
	if (nodeFire)
		stringstream(nodeFire.child_value())>>std::hex>>m_keyFire;

	return true;
}


void CTutorial2D::UpdateFrame()
{
	ProcessUserInput(.02f);
	UpdateWorld(.02f);
	m_pStarField->Update(.02f);

	if ( (rand()&63)==63)
	{
		//         
		std::string strTypeName = vecEnemyTypes[rand()%vecEnemyTypes.size()];

		//  ,     
		string strSprite = luabind::object_cast<string>(
			luabind::globals(myLuaState)[strTypeName]["spriteFile"]);
		int fSpeed = luabind::object_cast<float>(
			luabind::globals(myLuaState)[strTypeName]["speed"]);
		int iHealth = luabind::object_cast<int>(
			luabind::globals(myLuaState)[strTypeName]["health"]);

		//   wstring   
		std::wstring wName(strSprite.begin(), strSprite.end());
		vecEnemies.push_back(CEnemy(wName, iHealth, fSpeed));
	}
}

void CTutorial2D::ProcessUserInput(float fDeltaTime)
{
	int dx=0;
	if (GetAsyncKeyState(m_keyLeft)&32768)
		dx--; //  
	if (GetAsyncKeyState(m_keyRight)&32768)
		dx++; //  

#define SHIP_SPEED 400
	m_pPlayer->Scroll(dx*SHIP_SPEED*fDeltaTime, 0);

	static float fFireTimeOut=.0f;
	fFireTimeOut -= fDeltaTime;
	if (GetAsyncKeyState(m_keyFire)&32768 && fFireTimeOut<=0)
	{
		// 
		fFireTimeOut = .12f;
		CBullet bullet(m_pPlayer->Pos()-D3DXVECTOR2(0, 30), 
			D3DXVECTOR2(0, -300));
		vecBulletPlayer.push_back(bullet);
	}
}

void CTutorial2D::UpdateWorld(const float fDeltaTime)
{
	//   
	for_each(vecBulletPlayer.begin(), vecBulletPlayer.end(),
		bind2nd(mem_fun_ref(&CBullet::Update), fDeltaTime));

	for_each(vecBulletEnemy.begin(), vecBulletEnemy.end(),
		bind2nd(mem_fun_ref(&CBullet::Update), fDeltaTime));

	//  
	for_each(vecEnemies.begin(), vecEnemies.end(),
		bind2nd(mem_fun_ref(&CEnemy::Update), fDeltaTime));

	for_each(vecExplosions.begin(), vecExplosions.end(),
		bind2nd(mem_fun(&CExplosion::Update), fDeltaTime));

	//     ?
	for (size_t i=0; i<vecBulletEnemy.size(); i++)
	{
		if (vecBulletEnemy[i].TestCollision(*m_pPlayer))
		{
			//      
			m_pPlayer->DoDamage(vecBulletEnemy[i].Damage());
			vecBulletEnemy[i] = * vecBulletEnemy.rbegin();
			vecBulletEnemy.pop_back();
		}
		else if (vecBulletEnemy[i].Pos().y>300)
		{
			vecBulletEnemy[i] = * vecBulletEnemy.rbegin();
			vecBulletEnemy.pop_back();
		}
	}

	//    ?
	for (size_t i=0; i<vecEnemies.size(); i++)
	{
		//    
		if (vecEnemies[i].TestCollision(*m_pPlayer))
			vecEnemies[i].DoDamage(100); //  

		//     
		for (size_t j=0; j<vecBulletPlayer.size(); j++)
			if (vecBulletPlayer[j].TestCollision(vecEnemies[i]))
			{
				vecEnemies[i].DoDamage(vecBulletPlayer[j].Damage());
				vecBulletPlayer[j] = * vecBulletPlayer.rbegin();
				vecBulletPlayer.pop_back();
			}

			// ?
			if (!vecEnemies[i].IsAlive())
				vecExplosions.push_back( new CExplosion(vecEnemies[i].Pos()) );

			//      
			if (vecEnemies[i].Pos().y>400)
				vecEnemies[i].DoDamage(100000); // 
	}
	//   
	if (vecEnemies.size())
		vecEnemies.erase( remove_if(vecEnemies.begin(), vecEnemies.end(),
		not1(mem_fun_ref(&CEnemy::IsAlive))) , vecEnemies.end());

	for (size_t i=0; i<vecExplosions.size(); i++)
		if (vecExplosions[i]->IsFinished())
		{
			delete vecExplosions[i];
			vecExplosions[i] = *vecExplosions.rbegin();
			vecExplosions.pop_back();
		}
}
