#include "StdAfx.h"
#include "Tutorial3D.h"

using namespace std;

CTutorial3D::CTutorial3D(void)
{
	m_pPixelShaderPhong = m_pVertexShaderPhong = NULL;
	m_pTexDiffuse = NULL;
}

CTutorial3D::~CTutorial3D(void)
{
}

bool CTutorial3D::Init()
{
	RECT rc;

	GetClientRect(GetHwnd(), &rc);
	int iWndWidth = rc.right-rc.left;
	int iWndHeight = rc.bottom-rc.top;

	//  
	if (!Graphics::get().Init(GetHwnd(), iWndWidth, iWndHeight))
		return false;

	//   
	m_pVertexShaderPhong = Graphics::get().LoadShader("vs-phong.hlsl", "main", "vs_2_0");

	//   
	m_pPixelShaderPhong  = Graphics::get().LoadShader("ps-phong.hlsl", "pixel_shader_main", "ps_2_0");

	m_pTexDiffuse = Graphics::get().LoadTexture(L"Data/panel-01-diff.dds");

	m_modelTop.SetModel("Data/top.GAMEMODEL");
	m_modelMiddle.SetModel("Data/middle.GAMEMODEL");
	m_modelBottom.SetModel("Data/bottom.GAMEMODEL");

	m_modelTop.SetPos(D3DXVECTOR3(0, 16, 0));
	m_modelMiddle.SetPos(D3DXVECTOR3(0, 10, 0));
	m_modelBottom.SetPos(D3DXVECTOR3(0, 0, 0));
	
	m_modelMiddle.SetParent(&m_modelBottom);
	m_modelTop.SetParent(&m_modelMiddle);

	m_camera.SetLookAt(D3DXVECTOR3(25, 75, 75), D3DXVECTOR3(0, 30, 0), D3DXVECTOR3(0, 1, 0));
	m_camera.SetModPerpective(D3DX_PI*60/180, 1, 200, 800.0f/600.0f);

	return true;
}

void CTutorial3D::CleanUp()
{
	Graphics::get().Cleanup();
}

void CTutorial3D::Render()
{
	Graphics& gr = Graphics::get();
	if (!FAILED(gr.StartRender(0x00000000)))
	{
		//  -
		gr.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		gr.SetRenderState(D3DRS_LIGHTING, FALSE);
		gr.SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
		gr.SetRenderState(D3DRS_SHADEMODE, D3DSHADE_GOURAUD);
		gr.SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
		gr.SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);
		gr.SetRenderState(D3DRS_ZWRITEENABLE, TRUE);

		//   3-   
		gr.SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE);

		//  
		gr.SetPixelShader(m_pPixelShaderPhong);
		gr.SetVertexShader(m_pVertexShaderPhong);
		//    
		m_pVertexShaderPhong->SetMatrix("mViewProjection", &m_camera.GetViewProjection()/*matViewProj*/);
		m_pVertexShaderPhong->SetVector("vLight", &D3DXVECTOR4(250, 100, 100, 0));
		m_pVertexShaderPhong->SetVector("vCam", &m_camera.GetPos4()/*D3DXVECTOR4(eye.x, eye.y, eye.z, 0)*/);

		gr.SetTexture(0, m_pTexDiffuse);
		gr.SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_ANISOTROPIC);
		gr.SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_ANISOTROPIC);
		gr.SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
		
		m_pVertexShaderPhong->SetMatrix("mWorld", &m_modelTop.GetMatrix());
		gr.Draw(m_modelTop.GetMesh());
		m_pVertexShaderPhong->SetMatrix("mWorld", &m_modelMiddle.GetMatrix());
		gr.Draw(m_modelMiddle.GetMesh());
		m_pVertexShaderPhong->SetMatrix("mWorld", &m_modelBottom.GetMatrix());
		gr.Draw(m_modelBottom.GetMesh());

		//  
		gr.EndRender();
	}
}

void CTutorial3D::UpdateFrame(float fDeltaTime)
{
	D3DXMATRIX m;
	static float a=0.0f;
	a+=.03f;

	D3DXMatrixRotationZ(&m, -cos(a) + cos(a*3)*.2f);
	m_modelTop.SetRotation(m);

	D3DXMatrixRotationZ(&m, cos(a));
	m_modelMiddle.SetRotation(m);

	ProcessUserInput(fDeltaTime);
}

void CTutorial3D::ProcessUserInput(float fDeltaTime)
{
	float fDelta = fDeltaTime*100;
	
	if (GetAsyncKeyState(VK_LEFT)&32768)
		m_camera.SetPos(m_camera.GetPos() - m_camera.GetRight()*fDelta);

	if (GetAsyncKeyState(VK_RIGHT)&32768)
		m_camera.SetPos(m_camera.GetPos() + m_camera.GetRight()*fDelta);

	if (GetAsyncKeyState(VK_DOWN)&32768)
		m_camera.SetPos(m_camera.GetPos() - m_camera.GetForward()*fDelta);

	if (GetAsyncKeyState(VK_UP)&32768)
		m_camera.SetPos(m_camera.GetPos() + m_camera.GetForward()*fDelta);
}
